//////////////////////////////////////////////////////////////
//															//
//		Xors3D Engine. Advanced splatting shader			//
//		(c) 2010 Xors Team									//
//															//
//		www: http://xors3d.com								//
//		e-mail: support@xors3d.com							//
//															//
//////////////////////////////////////////////////////////////

const float blendRange = 0.1;

float4x4 worldViewProjMatrix : MATRIX_WORLDVIEWPROJ;
float4x4 worldViewMatrix     : MATRIX_WORLDVIEW;
float4x4 texture1Matrix      : MATRIX_TEXTURE0;
float4x4 texture2Matrix      : MATRIX_TEXTURE1;
float4x4 texture3Matrix      : MATRIX_TEXTURE2;
float4x4 texture4Matrix      : MATRIX_TEXTURE3;
float4x4 texture5Matrix      : MATRIX_TEXTURE4;
int      filtering           : TEXTURE_FILTERING;
int      anisotropyLevel     : ANISOTROPY_LEVEL;
bool     haveLightmap        : TERRAIN_LIGHTMAPPED;
float4   fogColor            : FOG_COLOR;
float3   fogRange            : FOG_RANGE;

texture	diffuseMap1 		: TEXTURE_0;
sampler samplerMap1 = sampler_state
{
    Texture       = <diffuseMap1>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture diffuseMap2 : TEXTURE_1;
sampler samplerMap2 = sampler_state
{
    Texture       = <diffuseMap2>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture diffuseMap3 : TEXTURE_2;
sampler samplerMap3 = sampler_state
{
    Texture       = <diffuseMap3>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture diffuseMap4 : TEXTURE_3;
sampler samplerMap4 = sampler_state
{
    Texture       = <diffuseMap4>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture diffuseMap5 : TEXTURE_4;
sampler samplerMap5 = sampler_state
{
    Texture       = <diffuseMap5>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture diffuseMap6 : TEXTURE_5;
sampler samplerMap6 = sampler_state
{
    Texture       = <diffuseMap6>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};

texture lightMap : TEXTURE_7;
sampler samplerLightmap = sampler_state
{
    Texture       = <lightMap>;
    ADDRESSU      = WRAP;
    ADDRESSV      = WRAP;
    ADDRESSW      = WRAP;
    MAGFILTER     = filtering;
    MINFILTER     = filtering;
    MIPFILTER     = filtering;
	MAXANISOTROPY = anisotropyLevel;
};
struct VSInput
{
   float4 position  : POSITION0;
   float2 texCoords : TEXCOORD0;
};
struct VSOutput2
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
};
struct VSOutput3
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float2 texCoords4 : TEXCOORD3;
};
struct VSOutput4
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float2 texCoords4 : TEXCOORD3;
	float2 texCoords5 : TEXCOORD4;
};
struct VSOutput5
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float2 texCoords4 : TEXCOORD3;
	float2 texCoords5 : TEXCOORD4;
	float2 texCoords6 : TEXCOORD5;
};
struct VSOutput2F
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float4 position2  : TEXCOORD5;
};
struct VSOutput3F
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float2 texCoords4 : TEXCOORD3;
	float4 position2  : TEXCOORD5;
};
struct VSOutput4F
{
	float4 position   : POSITION0;
	float2 texCoords1 : TEXCOORD0;
	float2 texCoords2 : TEXCOORD1;
	float2 texCoords3 : TEXCOORD2;
	float2 texCoords4 : TEXCOORD3;
	float2 texCoords5 : TEXCOORD4;
	float4 position2  : TEXCOORD5;
};
float4 LightMap(float2 texCoords)
{
	if(haveLightmap)
	{
		return tex2D(samplerLightmap, texCoords);
	}
	else
	{
		return float4(1.0f, 1.0f, 1.0f, 1.0f);
	}
}
float FogValue(float4 position)
{
	return clamp((fogRange.y - length(position.xyz)) / fogRange.z, 0.0f, 1.0f);
}
void VSMain2(in VSInput IN, out VSOutput2 OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = IN.texCoords;
}
void VSMain2F(in VSInput IN, out VSOutput2F OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.position2  = mul(IN.position,  worldViewMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = IN.texCoords;
}
void VSMain3(in VSInput IN, out VSOutput3 OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = mul(IN.texCoords, texture3Matrix);
	OUT.texCoords4 = IN.texCoords;
}
void VSMain3F(in VSInput IN, out VSOutput3F OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.position2  = mul(IN.position,  worldViewMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = mul(IN.texCoords, texture3Matrix);
	OUT.texCoords4 = IN.texCoords;
}
void VSMain4(in VSInput IN, out VSOutput4 OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = mul(IN.texCoords, texture3Matrix);
	OUT.texCoords4 = mul(IN.texCoords, texture4Matrix);
	OUT.texCoords5 = IN.texCoords;
}
void VSMain4F(in VSInput IN, out VSOutput4F OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.position2  = mul(IN.position,  worldViewMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = mul(IN.texCoords, texture3Matrix);
	OUT.texCoords4 = mul(IN.texCoords, texture4Matrix);
	OUT.texCoords5 = IN.texCoords;
}
void VSMain5(in VSInput IN, out VSOutput5 OUT)
{
	OUT.position   = mul(IN.position,  worldViewProjMatrix);
	OUT.texCoords1 = mul(IN.texCoords, texture1Matrix);
	OUT.texCoords2 = mul(IN.texCoords, texture2Matrix);
	OUT.texCoords3 = mul(IN.texCoords, texture3Matrix);
	OUT.texCoords4 = mul(IN.texCoords, texture4Matrix);
	OUT.texCoords5 = mul(IN.texCoords, texture5Matrix);
	OUT.texCoords6 = IN.texCoords;
}
float4 PSMain2(in VSOutput2 IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 mask     = tex2D(samplerMap3, IN.texCoords3);
	
	return lerp(diffuse2, diffuse1, saturate(smoothstep(mask.r - blendRange, mask.r + blendRange, 1.0 - diffuse2.a) * ceil(1.0 - mask.r) + floor(1.0 - mask.r)));
    /*if (diffuse2.a < mask.x - blendRange)     
    {
         return diffuse1;
    }
    else if (diffuse2.a > mask.x + blendRange)
    {
        return diffuse2;
    }
    else
    {
        //float f = saturate((lerp(diffuse2.a, 1 - blendRange, mask.x) - mask.x + blendRange) / (2 * blendRange));
		float f = (diffuse2.a - mask.x + blendRange) / (2 * blendRange);
        return lerp(diffuse1, diffuse2, f);
    }*/
}
float4 PSMain2F(in VSOutput2F IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 mask     = tex2D(samplerMap3, IN.texCoords3);
	float4 result   = (diffuse1 * mask.x + diffuse2 * mask.y) * LightMap(IN.texCoords3);
	float fog       = FogValue(IN.position2);
	return fogColor * (1.0f - fog) + result * fog;
}
float4 PSMain3(in VSOutput3 IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 diffuse3 = tex2D(samplerMap3, IN.texCoords3);
	float4 mask     = tex2D(samplerMap4, IN.texCoords4);	
	
	/*float4 blend23 = lerp(diffuse2, diffuse3, saturate(smoothstep(mask.g - blendRange, mask.g + blendRange, diffuse3.a) * ceil(1.0 - mask.g) + floor(1.0 - mask.g)));	
	return lerp(diffuse1, blend23, saturate(smoothstep(mask.r - blendRange, mask.r + blendRange, diffuse2.a) * ceil(1.0 - mask.r) + floor(1.0 - mask.r)));*/
	
	float4 blend1_2 = lerp(diffuse2, diffuse1, saturate(smoothstep(mask.r - blendRange, mask.r + blendRange, diffuse2.a) * ceil(1.0 - mask.r) + floor(1.0 - mask.r)));	
	return lerp(diffuse3, blend1_2, saturate(smoothstep(mask.g - blendRange, mask.g + blendRange, 1.0 - diffuse3.a) * ceil(1.0 - mask.g) + floor(1.0 - mask.g)));
	
	//return (diffuse1 * mask.x + diffuse2 * mask.y + diffuse3 * mask.z) * LightMap(IN.texCoords4);
}
float4 PSMain3F(in VSOutput3F IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 diffuse3 = tex2D(samplerMap3, IN.texCoords3);
	float4 mask     = tex2D(samplerMap4, IN.texCoords4);
	float4 result   = (diffuse1 * mask.x + diffuse2 * mask.y + diffuse3 * mask.z) * LightMap(IN.texCoords4);
	float fog       = FogValue(IN.position2);
	return fogColor * (1.0f - fog) + result * fog;
}
float4 PSMain4(in VSOutput4 IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 diffuse3 = tex2D(samplerMap3, IN.texCoords3);
	float4 diffuse4 = tex2D(samplerMap4, IN.texCoords4);
	float4 mask     = tex2D(samplerMap5, IN.texCoords5);
	
	float4 blend1_2 = lerp(diffuse2, diffuse1, saturate(smoothstep(mask.r - blendRange, mask.r + blendRange, diffuse2.a) * ceil(1.0 - mask.r) + floor(1.0 - mask.r)));	
	float4 blend12_3 = lerp(diffuse3, blend1_2, saturate(smoothstep(mask.g - blendRange, mask.g + blendRange, 1.0 - diffuse3.a) * ceil(1.0 - mask.g) + floor(1.0 - mask.g)));
	return lerp(diffuse4, blend12_3, saturate(smoothstep(mask.b - blendRange, mask.b + blendRange, 1.0 - diffuse4.a) * ceil(1.0 - mask.b) + floor(1.0 - mask.b)));
	
	//return (diffuse1 * mask.x + diffuse2 * mask.y + diffuse3 * mask.z + diffuse4 * mask.w) * LightMap(IN.texCoords5);
}
float4 PSMain4F(in VSOutput4F IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 diffuse3 = tex2D(samplerMap3, IN.texCoords3);
	float4 diffuse4 = tex2D(samplerMap4, IN.texCoords4);
	float4 mask     = tex2D(samplerMap5, IN.texCoords5);
	float4 result   = (diffuse1 * mask.x + diffuse2 * mask.y + diffuse3 * mask.z + diffuse4 * mask.w) * LightMap(IN.texCoords5);
	float fog       = FogValue(IN.position2);
	return fogColor * (1.0f - fog)+ result * fog;
}
float4 PSMain5(in VSOutput5 IN) : COLOR
{
	float4 diffuse1 = tex2D(samplerMap1, IN.texCoords1);
	float4 diffuse2 = tex2D(samplerMap2, IN.texCoords2);
	float4 diffuse3 = tex2D(samplerMap3, IN.texCoords3);
	float4 diffuse4 = tex2D(samplerMap4, IN.texCoords4);
	float4 diffuse5 = tex2D(samplerMap5, IN.texCoords5);
	float4 mask     = tex2D(samplerMap6, IN.texCoords6);
	
	float4 blend1_2 = lerp(diffuse2, diffuse1, saturate(smoothstep(mask.r - blendRange, mask.r + blendRange, diffuse2.a) * ceil(1.0 - mask.r) + floor(1.0 - mask.r)));	
	float4 blend12_3 = lerp(diffuse3, blend1_2, saturate(smoothstep(mask.g - blendRange, mask.g + blendRange, 1.0 - diffuse3.a) * ceil(1.0 - mask.g) + floor(1.0 - mask.g)));
	float4 blend123 = lerp(diffuse4, blend12_3, saturate(smoothstep(mask.b - blendRange, mask.b + blendRange, 1.0 - diffuse4.a) * ceil(1.0 - mask.b) + floor(1.0 - mask.b)));
	return lerp(diffuse5, blend123, saturate(smoothstep(mask.a - blendRange, mask.a + blendRange, 1.0 - diffuse5.a) * ceil(1.0 - mask.a) + floor(1.0 - mask.a)));
}
technique Splatting2
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain2();
		PixelShader      = compile ps_2_0 PSMain2();
	}
}
technique Splatting3
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain3();
		PixelShader      = compile ps_2_0 PSMain3();
	}
}
technique Splatting4
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain4();
		PixelShader      = compile ps_2_0 PSMain4();
	}
}
technique Splatting5
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_3_0 VSMain5();
		PixelShader      = compile ps_3_0 PSMain5();
	}
}
technique Splatting2Fog
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain2F();
		PixelShader      = compile ps_2_0 PSMain2F();
	}
}
technique Splatting3Fog
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain3F();
		PixelShader      = compile ps_2_0 PSMain3F();
	}
}
technique Splatting4Fog
{
	pass p0
	{
		AlphaBlendEnable = false;
		VertexShader     = compile vs_2_0 VSMain4F();
		PixelShader      = compile ps_2_0 PSMain4F();
	}
}